@@ -1,3 +1,11 @@
+# Copyright (c) 2019-2020, RTE (https://www.rte-france.com)
+# See AUTHORS.txt
+# This Source Code Form is subject to the terms of the Mozilla Public License, version 2.0.
+# If a copy of the Mozilla Public License, version 2.0 was not distributed with this file,
+# you can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-License-Identifier: MPL-2.0
+# This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
+
 """
 This module defines the :class:`Environment` the higher level representation of the world with which an
 :class:`grid2op.BaseAgent` will interact.
@@ -45,6 +53,7 @@ Example (adapted from gym documentation available at
 import numpy as np
 import os
 import copy
+import pdb
 
 from grid2op.Action import ActionSpace, BaseAction, TopologyAction, DontAct, CompleteAction
 from grid2op.Exceptions import *
@@ -382,9 +391,10 @@ class Environment(_BasicEnv):
         # first injections given)
         self._reset_maintenance()
         do_nothing = self.helper_action_env({})
-        *_, fail_to_start, _ = self.step(do_nothing)
+        *_, fail_to_start, info = self.step(do_nothing)
         if fail_to_start:
-            raise Grid2OpException("Impossible to initialize the powergrid, the powerflow diverge at iteration 0.")
+            raise Grid2OpException("Impossible to initialize the powergrid, the powerflow diverge at iteration 0. "
+                                   "Available information are: {}".format(info))
 
         # test the backend returns object of the proper size
         self.backend.assert_grid_correct_after_powerflow()
@@ -506,9 +516,9 @@ class Environment(_BasicEnv):
     def attach_renderer(self, graph_layout=None):
         if self.viewer is not None:
             return
-        graph_layout = self.graph_layout if graph_layout is None else graph_layout
         if graph_layout is not None:
-            self.viewer = PlotPyGame(graph_layout, observation_space=self.helper_observation)
+            self.viewer = PlotPyGame(observation_space=self.helper_observation,
+                                     substation_layout=graph_layout)
             self.viewer.reset(self)
         else:
             raise PlotError("No layout are available for the powergrid. Renderer is not possible.")
@@ -534,8 +544,15 @@ class Environment(_BasicEnv):
         if self._thermal_limit_a is not None:
             self.backend.set_thermal_limit(self._thermal_limit_a)
 
+        # TODO this is super weird!!!!
+        # self.gen_downtime = self.gen_min_downtime + 1
+        # self.gen_uptime = self.gen_min_uptime + 1
         do_nothing = self.helper_action_env({})
-        self.step(do_nothing)
+        *_, fail_to_start, info = self.step(do_nothing)
+        if fail_to_start:
+            raise Grid2OpException("Impossible to initialize the powergrid, the powerflow diverge at iteration 0. "
+                                   "Available information are: {}".format(info))
+
         # test the backend returns object of the proper size
         self.backend.assert_grid_correct_after_powerflow()
 
@@ -594,13 +611,16 @@ class Environment(_BasicEnv):
                                          self.backend.name_line, self.backend.name_sub,
                                          names_chronics_to_backend=self.names_chronics_to_backend)
         self.current_obs = None
+        self.env_modification = None
         self._reset_maintenance()
         self._reset_redispatching()
+        self._reset_vectors_and_timings()  # it need to be done BEFORE to prevent cascading failure when there has been
         self.reset_grid()
         if self.viewer is not None:
             self.viewer.reset(self)
         # if True, then it will not disconnect lines above their thermal limits
-        self._reset_vectors_and_timings()
+        self._reset_vectors_and_timings()  # and it needs to be done AFTER to have proper timings at tbe beginning
+        # TODO add test above: fake a cascading failure, do a reset, check that it can be loaded
         # reset the opponent
         self.oppSpace.reset()
         return self.get_obs()
