@@ -1,24 +1,20 @@
-import os
-import sys
-import unittest
-import datetime
+# Copyright (c) 2019-2020, RTE (https://www.rte-france.com)
+# See AUTHORS.txt
+# This Source Code Form is subject to the terms of the Mozilla Public License, version 2.0.
+# If a copy of the Mozilla Public License, version 2.0 was not distributed with this file,
+# you can obtain one at http://mozilla.org/MPL/2.0/.
+# SPDX-License-Identifier: MPL-2.0
+# This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
+
 import tempfile
-import time
-import numpy as np
 import pdb
 
 from grid2op.tests.helper_path_test import *
 
 from grid2op.Exceptions import *
-from grid2op.Observation import ObservationSpace, CompleteObservation, ObsEnv
 from grid2op.Chronics import Multifolder
-from grid2op.Exceptions import *
-from grid2op.Action import ActionSpace
-from grid2op.Rules import RulesChecker
 from grid2op.Reward import L2RPNReward
-from grid2op.Parameters import Parameters
 from grid2op.Backend import PandaPowerBackend
-from grid2op.Environment import Environment
 from grid2op.Runner import Runner
 from grid2op.EpisodeData import EpisodeData
 
